from odoo import exceptions, fields, models


# ExpressionBuilder
class ExpressionBuilder(models.TransientModel):
    _name = "expression.builder.wizard"
    _description = "Expression Builder Wizard"

    # the substitution the expression builder is working on
    substitution_id = fields.Many2one(
        comodel_name="redner.substitution",
        string="Substitution",
        help="The Redner Substitution on which we assign the value",
        required=True,
    )
    # the expression field is concatened with the fields values
    # when the wizard action is completed, this field is
    # written in the substitution value
    expression = fields.Char(
        string="Expression",
        help=(
            "Stores the substitution value the builder is working on."
            " (example: 'partner_id.name')"
        ),
    )

    suggested_fields = fields.One2many(
        comodel_name="expression.builder.field",
        inverse_name="wizard_id",
        string="Suggested Fields",
        help="The suggested fields, based on the substitution converter value",
    )

    def get_fields(self, model, converter):
        # Get all field records for the model
        vals_list = self.env["ir.model.fields"].search([("model", "=", model._name)])

        # Filter fields based on the converter type
        if converter == "relation-to-many":
            vals_list = [f for f in vals_list if f.ttype == "many2many"]
        elif converter == "field":
            vals_list = [f for f in vals_list if f.ttype != "many2many"]
        else:
            vals_list = []
        return [
            {
                "field_name": f.field_description,
                "field_technical_name": f.name,
                "field_type": f.ttype,
                "field_comodel_name": f.relation,
            }
            for f in vals_list
        ]

    def action_cancel(self):
        # custom cancel, unlinks the fields, the wizard itself
        # and close window
        self.unlink()
        return {"type": "ir.actions.act_window_close"}

    def action_save(self):
        selected_count = 0
        selected_field = False
        for f in self.suggested_fields:
            if f.selected:
                selected_count += 1
                selected_field = f

        if selected_count > 1:
            raise exceptions.UserError(
                "You cannot select more than one field to build the expression"
            )

        elif selected_count == 0:
            raise exceptions.UserError(
                "You have to select one field to build the expression"
            )
        elif selected_field.field_type in ["many2one", "one2many"]:
            # in this case we do not close the window.
            # instead we refresh the wizard by showing the
            # fields 1 field deeper
            # and we update the expression field
            if self.expression or self.expression != "":
                self.expression += "." + selected_field.field_technical_name
            else:
                self.expression = selected_field.field_technical_name

            # get comodel
            comodel = self.env.get(selected_field.field_comodel_name)
            if comodel is None:
                return
            vals_list = self.get_fields(comodel, self.substitution_id.converter)
            for val in vals_list:
                val["wizard_id"] = self.id

            # remove previous fields
            self.suggested_fields.unlink()
            # add and create the next fields
            fields_list = self.env["expression.builder.field"].create(vals_list)
            self.suggested_fields = [(6, 0, fields_list.ids)]
            # stay on updated wizard
            return {
                "type": "ir.actions.act_window",
                "res_model": "expression.builder.wizard",
                "view_mode": "form",
                "res_id": self.id,
                "target": "new",
            }
        # if the selected field is not relationnal:
        if self.expression or self.expression != "":
            # expression has already a value
            self.substitution_id.write(
                {
                    "value": self.expression
                    + "."
                    + selected_field.field_technical_name,
                }
            )
        else:
            # expression is empty
            self.substitution_id.write(
                {
                    "value": selected_field.field_technical_name,
                }
            )
        # at the end, we unlink all of the temp data
        # and close the window
        return self.action_cancel()
